@@ -1,7 +1,7 @@
 import csv
 import logging
 import os
-from os.path import basename, join
+from os.path import basename, join, splitext
 
 HERE = os.path.dirname(__file__)
 logging.basicConfig(level=logging.DEBUG)
@@ -22,6 +22,8 @@ class CSVProcessor(object):
         self.fk_mapping = {}
         self.lines = 0
         self.is_moved = set()
+        self.existing_records = {}
+        self.existing_records_without_id = {}
 
     def get_target_columns(self, filepaths):
         """ Compute target columns with source columns + mapping
@@ -35,10 +37,11 @@ class CSVProcessor(object):
             for source_column in source_columns + ['_']:
                 mapping = self.mapping.get_targets('%s.%s' % (source_table, source_column))
                 # no mapping found, we warn the user
-                if mapping in (None, '__copy__'):
+                if mapping is None:
                     origin = source_table + '.' + source_column
-                    if source_column != '_':
-                        LOG.warn('No mapping definition found for column %s', origin)
+                    LOG.warn('No mapping definition found for column %s', origin)
+                    continue
+                if mapping == '__copy__':
                     continue
                 elif mapping in (False, '__forget__'):
                     continue
@@ -51,12 +54,52 @@ class CSVProcessor(object):
                                for k, v in self.target_columns.items()}
         return self.target_columns
 
+    def set_existing_data(self, existing_records):
+        """let the existing data be accessible during processing
+        """
+        self.existing_records = existing_records
+        # the same without ids
+        self.existing_records_without_id = {
+            table: [{k: str(v) for k, v in nt.iteritems() if k != 'id'} for nt in existing]
+            for table, existing in existing_records.iteritems()
+        }
+
+    def reorder_with_discriminators(self, tables):
+        """ Reorder the filepaths based on tables pointed by discriminators
+        (if they are fk)
+        """
+        # get the list of tables pointed by discriminators
+        discriminator_tables = set()
+        for table, columns in self.mapping.discriminators.iteritems():
+            for column in columns:
+                field = table + '.' + column
+                if field in self.fk2update:
+                    discriminator_tables.add(self.fk2update[field])
+        # remove them from the initial tables
+        tables = [t for t in tables if t not in discriminator_tables]
+        # reorder the small set with a very basic algorithm:
+        # put at left those without fk discriminator, right others
+        ordered_tables = []
+        for table in discriminator_tables:
+            for column in self.mapping.discriminators.get(table, ()):
+                field = table + '.' + column
+                if table in ordered_tables:
+                    continue
+                if field in self.fk2update:
+                    ordered_tables.append(table)
+                else:
+                    ordered_tables.insert(0, table)
+        # append the two lists
+        ordered_tables += tables
+        return ordered_tables
+
     def process(self, source_dir, source_filenames, target_dir,
-                target_connection=None, existing_records=None):
+                target_connection=None):
         """ The main processing method
         """
         # compute the target columns
         filepaths = [join(source_dir, source_filename) for source_filename in source_filenames]
+        source_tables = [splitext(basename(path))[0] for path in filepaths]
         self.target_columns = self.get_target_columns(filepaths)
         # load discriminator values for target tables
         # TODO
@@ -89,9 +132,14 @@ class CSVProcessor(object):
         for writer in self.updatewriters.values():
             writer.writeheader()
         LOG.info(u"Processing CSV files...")
-        for source_filename in source_filenames:
-            source_filepath = join(source_dir, source_filename)
-            self.process_one(source_filepath, target_connection, existing_records)
+        # We should first reorder the processing so that tables pointed to by
+        # discriminator values which are fk be processed first. This is not the
+        # most common case, but otherwise offsetting these values may fail,
+        # leading to unwanted matching and unwanted merge.
+        ordered_tables = self.reorder_with_discriminators(source_tables)
+        ordered_paths = [join(source_dir, table + '.csv') for table in ordered_tables]
+        for source_filepath in ordered_paths:
+            self.process_one(source_filepath, target_connection)
         # close files
         for target_file in target_files.values():
             target_file.close()
@@ -114,7 +162,7 @@ class CSVProcessor(object):
         LOG.info(u"Postprocessing CSV files...")
         for filename in target_filenames.values():
             filepath = join(target_dir, filename)
-            self.postprocess_one(filepath, existing_records)
+            self.postprocess_one(filepath)
         for f in target2_files.values():
             f.close()
 
@@ -133,16 +181,17 @@ class CSVProcessor(object):
             writer.writeheader()
         for filename in update_filenames.values():
             filepath = join(target_dir, filename)
-            self.postprocess_one(filepath, existing_records)
+            self.postprocess_one(filepath)
         # close files
         for f in update2_files.values():
             f.close()
 
     def process_one(self, source_filepath,
-                    target_connection=None, existing_records=None):
+                    target_connection=None):
         """ Process one csv file
+        The fk_mapping should not be read in this method. Only during postprocessing,
+        Because the processing order is not determined (unordered dicts)
         """
-        existing_records = existing_records or {}
         source_table = basename(source_filepath).rsplit('.', 1)[0]
         with open(source_filepath, 'rb') as source_csv:
             reader = csv.DictReader(source_csv, delimiter=',')
@@ -190,9 +239,8 @@ class CSVProcessor(object):
                     discriminators = self.mapping.discriminators.get(table)
                     # if the line exists in the target db, we don't offset and write to update file
                     # (we recognize by matching the dict of discriminator values against existing)
-                    existing = existing_records.get(table, [])
-                    existing_without_id = [{k: str(v) for k, v in nt.iteritems() if k != 'id'}
-                                           for nt in existing]
+                    existing = self.existing_records.get(table, [])
+                    existing_without_id = self.existing_records_without_id.get(table, [])
                     discriminator_values = {d: target_row[d] for d in (discriminators or [])}
                     # before matching existing, we should fix the discriminator_values which are fk
                     # FIXME refactor and merge with the code in postprocess
@@ -200,9 +248,11 @@ class CSVProcessor(object):
                         fk_table = self.fk2update.get(table + '.' + key)
                         if value and fk_table:
                             value = int(value)
+                            # this is BROKEN because it needs the fk_table to be processed before.
                             if value in self.fk_mapping.get(fk_table, []):
                                 discriminator_values[key] = str(
                                     self.fk_mapping[fk_table].get(value, value))
+                    # save the mapping between source id and existing id
                     if (discriminators
                             and 'id' in target_row
                             and all(discriminator_values.values())
@@ -248,10 +298,9 @@ class CSVProcessor(object):
                         # otherwise write the target csv line
                         self.writers[table].writerow(target_row)
 
-    def postprocess_one(self, target_filepath, existing_records=None):
+    def postprocess_one(self, target_filepath):
         """ Postprocess one target csv file
         """
-        existing_records = existing_records or {}
         table = basename(target_filepath).rsplit('.', 2)[0]
         with open(target_filepath, 'rb') as target_csv:
             reader = csv.DictReader(target_csv, delimiter=',')
@@ -278,13 +327,10 @@ class CSVProcessor(object):
                 # don't write m2m lines if they exist in the target
                 # FIXME: refactor these 4 lines with those from process_one()?
                 discriminators = self.mapping.discriminators.get(table)
-                existing = existing_records.get(table, [])
-                existing_without_id = [{k: v for k, v in nt.iteritems() if k != 'id'}
-                                       for nt in existing]
-                discriminator_values = {d: postprocessed_row[d] for d in (discriminators or [])}
-                if ('id' in postprocessed_row
-                        or {k: int(v) for k, v in discriminator_values.iteritems()}
-                        not in existing_without_id):
+                existing_without_id = self.existing_records_without_id.get(table, [])
+                discriminator_values = {d: str(postprocessed_row[d])
+                                        for d in (discriminators or [])}
+                if 'id' in postprocessed_row or discriminator_values not in existing_without_id:
                     self.writers[table].writerow(postprocessed_row)
 
     def update_one(self, filepath, connection):
